/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef _PHOENIX_
#define _PHOENIX_

// System includes 

#ifdef GCC
#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#define _THREAD_SAFE
#define _REENTRANT
#endif

#ifdef LINUX
#define X86SERIAL
#endif

#ifdef FREEBSD
#define X86SERIAL
#endif

#ifdef CYGWIN
#define X86SERIAL
#endif

#ifdef WIN32
#include <windows.h>
#include <process.h>

typedef long int ssize_t;
typedef int mode_t;

#include <time.h>
#pragma warning ( disable : 4200)
#include <io.h>

#define getpid GetCurrentProcessId
#define usleep(x)	Sleep(x/1000)
#define sleep(x)	Sleep((x)*1000)
#define ONE_SECOND	1000

#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#ifdef POSIX
#define HAS_FORK
#ifndef NO_THREADS
#include <pthread.h>
#define THREAD_SAFE
#include <unistd.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>
#endif
#include <sys/time.h>
#include <sys/types.h>
#ifndef DARWIN
#include <sys/poll.h>
#endif
#include <sys/times.h>
#endif	

#include <sys/stat.h>
#include <assert.h>
#include <fcntl.h>


// Project types

#ifndef Boolean_DEFINED
#define Boolean_DEFINED
typedef unsigned char Boolean;
#endif

#ifndef BYTE_DEFINED
#define BYTE_DEFINED
typedef unsigned char BYTE;
#endif

#ifndef PVOID_DEFINED
#define PVOID_DEFINED
typedef void *PVOID;
#endif

// Standard defines

#ifndef TRUE
#define TRUE	1
#endif
#ifndef FALSE
#define FALSE 	0
#endif

#ifndef MAX
#define MAX(a,b) (((a)>(b))?(a):(b))
#endif
#ifndef MIN
#define MIN(a,b) (((a)<(b))?(a):(b))
#endif

#define MAX_STRING 4096

// Phoenix memory management
void *	operator new (size_t);
void operator delete (void *);

void *	_phmalloc(unsigned int cb);
void	_phmfree(void *p);
void *	_phmcalloc(unsigned int num, unsigned int cbelement);
void *	_phmrealloc(void *p, unsigned int cbnew);
char *	_phmstrdup(char *str);


// Define this for memory debugging
// #define DEBUG_MEMORY

#ifdef DEBUG_MEMORY
void *	phmalloc_dbg(unsigned int cb, char *file, int line);
void	phmfree_dbg(void *p, char *file, int line);
void *	phmcalloc_dbg(unsigned int num, unsigned int cbelement, char *file, int line);
void *	phmrealloc_dbg(void *p, unsigned int cbnew, char *file, int line);
char *	phmstrdup_dbg(char *str, char *file, int line);

#define phmalloc(cb)		phmalloc_dbg(cb, __FILE__, __LINE__)
#define phmfree(p)		phmfree_dbg(p, __FILE__, __LINE__)
#define phmcalloc(num, cb)	phmcalloc_dbg(num, cb, __FILE__, __LINE__)
#define phmrealloc(p,cbnew)	phmrealloc_dbg(p,cbnew,__FILE__, __LINE__)
#define phmstrdup(str)		phmstrdup_dbg(str, __FILE__, __LINE__)
#else
#define phmalloc(cb)		_phmalloc(cb)
#define phmfree(p)		_phmfree(p)
#define phmcalloc(num, cb)	_phmcalloc(num, cb)
#define phmrealloc(p,cbnew)	_phmrealloc(p,cbnew)
#define phmstrdup(str)		_phmstrdup(str)
#endif

int 	phmtotal(void);
int 	phmcount(void);
int 	phmsystotal(void);
int 	phmfrag(void);

//
// Project Modules
//

#ifdef WIN32
#include <malloc.h>
typedef unsigned long 	FOFFSET;
#define VSNPRINTF	_vsnprintf
#define ALLOCA(cb)	_alloca(cb)
#else
typedef off_t		FOFFSET;
#define VSNPRINTF	vsnprintf
#define ALLOCA(cb)	alloca(cb)
#endif

#include "mmutex.h"
#include "msql.h"
#include "mutils.h"
#include "mtoken.h"
#include "mfile.h"
#include "mnode.h"
#include "msort.h"
#include "msocket.h"
#include "mwww.h"

#ifndef NO_MARRAY
#include "marray.h"
#include "memheap.h"
#include "mnodelist.h"
#endif


#include "mlist.h"
#include "mthread.h"
#include "mserver.h"
#include "mcircular.h"
#include "mfblkfile.h"
#include "mfprst.h"
#include "mfkbd.h"
#include "mftimer.h"

#include "mtreeint.h"
#include "mavl.h"
#include "mhist.h"
#ifndef NO_MARRAY
 #include "mconfig.h"
 #include "mhash.h"
#endif
#include "mplist.h"

#ifndef NO_PLUGIN
#include "mplugin.h"
#endif

#ifndef NO_GETOPTION
#include "getoption.h"
#endif

#ifdef LINK_KIT
#define MAIN(module)		module##_main
#else
#define MAIN(module)	main
#endif

#define LINKMAIN(module)	module##_main
//
// Debugging aids
//
#ifdef TRACE_DEBUG
#ifndef NTRACE_DO_BUG
#define TRACE_DO_BUG
#endif
#ifndef NTRACE_DO_HEX
#define TRACE_DO_HEX
#endif
#ifndef NTRACE_DO_VAL
#define TRACE_DO_VAL
#endif
#ifndef NTRACE_DO_STR
#define TRACE_DO_STR
#endif
#ifndef NTRACE_DO_FUNCT
#define TRACE_DO_FUNCT
#endif
#ifndef NTRACE_DO_EXIT
#define TRACE_DO_EXIT
#endif
#ifndef NTRACE_DO_ASSERTS
#define TRACE_DO_ASSERTS
#endif
#endif

#ifdef TRACE_DO_BUG
#define TRACE_BUG	mferr.mfprintf("%s,%d [%d]\n",__FILE__, __LINE__,getpid()); mferr.flush();
#else
#define TRACE_BUG
#endif

#ifdef TRACE_DO_HEX
#define TRACE_HEX(p)	mferr.mfprintf("%s,%d [%d] (%x)\n", __FILE__,__LINE__,getpid(),(unsigned)p); mferr.flush();
#else
#define TRACE_HEX(p)
#endif

#ifdef TRACE_DO_VAL
#define TRACE_VAL(p)	mferr.mfprintf( "%s,%d [%d] (%d)\n", __FILE__,__LINE__,getpid(),p); mferr.flush();
#else
#define TRACE_VAL(p)
#endif

#ifdef TRACE_DO_STR
#define TRACE_STR(p)	mferr.mfprintf("%s,%d [%d] (%s)\n", __FILE__,__LINE__,getpid(),p); mferr.flush();
#define TRACE_STRVAL(p,v) mferr.mfprintf("%s,%d [%d] %s = %d\n", __FILE__,__LINE__,getpid(),p,v); mferr.flush();
#define TRACE_STRSTR(p,s) mferr.mfprintf("%s,%d [%d] [%s][%s]\n", __FILE__,__LINE__,getpid(),p,s); mferr.flush();
#else
#define TRACE_STR(p)
#define TRACE_STRVAL(p,v) 
#define TRACE_STRSTR(p,s)
#endif

#ifdef TRACE_DO_FUNCT
#define TRACE_FUNCT(p)	mferr.mfprintf( "%s,%d [%d] (%s)\n", __FILE__,__LINE__,getpid(),p); mferr.flush();
#else
#define TRACE_FUNCT(p)
#endif

#ifdef TRACE_DO_EXIT
#define TRACE_EXIT(p) trace_exit(__FILE__, __LINE__, p)
#else
#define TRACE_EXIT(p)
#endif

#ifdef TRACE_DO_ASSERTS
#define TRACE_ASSERT(exp) if(!(exp)) trace_exit(__FILE__,__LINE__, #exp)	
#define TRACE_TEST(exp) if(!(exp)) mferr.mfprintf("%s:%d expression failed: %s\n", __FILE__,__LINE__,#exp)
#else
#define TRACE_ASSERT(exp)
#define TRACE_TEST(exp)
#endif

#ifdef TRACE_DO_MARKER
#define TRACE_MARKER mferr.mfprintf("%s:%d\n",__FILE__, __LINE__);
#endif

#define TRACE_ERROR(str) mferr.mfprintf("%s: %s,%n\n", str, __FILE__, __LINE__)
#endif
