/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef _CIRCULAR_
#define _CIRCULAR_

/** A circular buffer template, like a fifo */
template <class TYPE> class MTCircleBuffer
{
	protected:
	THREAD_SAFE_MTX	

	TYPE *	m_buffer;
	int 	m_ndxHead;
	int 	m_ndxTail;
	int 	m_count;
	int	m_max;

	public:

	MTCircleBuffer(int num)
	{
		int cb = sizeof(TYPE) * num;
		m_buffer = (TYPE *) phmalloc(cb);

		m_ndxHead = 0;
		m_ndxTail = 0;
		m_count = 0;
		m_max = num;
	}

	~MTCircleBuffer()
	{
		THREAD_SAFE_LOCK;
		phmfree((void *)m_buffer);
	}

	/** Put an item into the buffer */
	Boolean Put(TYPE t)
	{
		THREAD_SAFE_LOCK;
		if(m_count < m_max)
		{
			m_buffer[m_ndxHead++]=t;

			if(m_ndxHead >= m_max)
				m_ndxHead=0;
			m_count++;
			return TRUE;
		}
		return FALSE;
	}

	/** Get an item from the buffer */
	Boolean Get(TYPE *pt)
	{
		THREAD_SAFE_LOCK;
		if(pt && (m_count > 0))
		{
			*pt = m_buffer[m_ndxTail++];
			if(m_ndxTail >= m_max)
				m_ndxTail=0;
			m_count--;
			return TRUE;
			
		}
		return FALSE;
	}
};

#endif
