/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef _MFBLKFILE_
#define _MFBLKFILE_

#define	MFBLK_DIRTY	0x001
#define	MFBLK_USED	0x002
#define	MFBLK_USER	0x004
#define MFBLK_LOCKED	0x010
#define MFBLK_INMEM	0x200
#define MFBLK_INTERNAL	0x400
#define MFBLK_MAGIC	0x42420000
#define MFBLK_MAGICMASK	0xFFFF0000


typedef unsigned long 	HBLK;
typedef unsigned long	UINT32;

/** Block header for each file entry */
struct _mfblk_file // 32 bytes long
{
	/** Whole size of block, inclusive */
        UINT32                  usize;          
	/** System flags */
        UINT32                  uflags;       
   	/** user property value, in free blocks prev */
        UINT32                  uprop;       
 	/** user defined type, in free blocs next */
        UINT32                  utype;         
};

struct _mfblk;

/** Memory block header for each block in memory */
typedef struct _mfblk
{
	/** saved in file */
	UINT32			uflags;		
	/** saved in file	*/	
	UINT32			usize;		
	/** saved in file; */
	UINT32			uprop;		
	/** saved in file */
	UINT32			utype;	
	class MFBlkfile	*	blkf;
	/** next in list */
	struct _mfblk 	*	pnext;	
	/** prev in list */
	struct _mfblk 	*	pprev;	
	/** my handle in system */
	HBLK			handle;		
	/** My lock count */
	UINT32			lockcount;
	/** Data */
	char			data[4];		
// file
}BLK, *PBLK;

// 64 master list items
#ifndef MAX_MASTLIST
#define MAX_MASTLIST	64	
#endif

/** File header, begins all block files */
struct  _mfblk_header
{
	/** signature of file */
	char	sig[4];			
	/** size of header block, number of list items */
	UINT32	blksize;		
	/** Offset to beginning of file '0' */
	UINT32	offset;			
	/** List of free blocks */
	HBLK	freelist;		
	/** master list of handles */
	HBLK	master[MAX_MASTLIST];	
}; 

/** Memory reude strategy */
enum BLK_REUSE
{
	BLK_NOREUSE,
        BLK_BESTFIT,
        BLK_WORSTFIT,
        BLK_FIRSTFIT
};  

class MFPersistent;
/** A file class which manages blocks within a file. It looks more like a 
handle based memory allocator. Memory in a file is allocated in both RAM
and in the file. A pointer is then returned to the memory. When the memory 
freed, it is written to the file.

There are MAX_MASTLIST handes in the file header. An object can be allocated
with in the file, and its handle stored as a master handle. This is sort of a 
bootstrap mechanism.

Blocks cn be manipulated either as a pointer to its block, or as a pointer to
its data. block pointers are a bit more efficient, data pointers look cleaner.
*/
class MFBlkfile
{
	protected:

	/** Ofset into parent file */
	unsigned long		m_foffset;
	/** Locked for a long operation */
	Boolean			m_flocked;
	/** Internal file object */
	MFile			*m_file;
	/** File header */
	_mfblk_header		m_header;
	/** List of used blocks */
	PBLK			m_list;	
	
	/** internal Functions for dealing with m_list */
	virtual	void		mlistadd(PBLK pblk);
	/** internal Functions for dealing with m_list */
	virtual	void		mlistremove(PBLK pblk);
	/** internal Functions for dealing with m_list */
	virtual	void		mlistclearall(void);
	/** internal Functions for dealing with m_list */
	virtual	void		mlistdelete(PBLK pblk);

	/** internal Memory alloc */
	virtual	PBLK		blkmemalloc(size_t size, HBLK handle);	// allocate memory
	/** internal Memory free */
	virtual void		blkmemfree(PBLK pblk);			// free memory allocated

	/** get maximum master list entries */
	virtual	int		getmaxmaster(void);			

	/** Add block to free list */
	virtual	void		flistadd(PBLK pblk);
	/** Remove block from free list */
	virtual	void		flistremove(PBLK pblk);
	/** Get pointer to a free block, according to method */
	virtual PBLK		flistfindfree(size_t size, enum BLK_REUSE method);
	/** Reuse a block handle */
	virtual PBLK		flistreuse(HBLK handle);
	

	/** Initialize a block */
	virtual PBLK		blkinit(PBLK pblk, size_t size, HBLK handle);
	/** Allocate a block */
	virtual	PBLK		blkalloc(size_t size);
	/** Free a block */
	virtual	void		blkfree(PBLK pblk);			// remove block, add to free list
	/** Get a block from a handle */
	virtual	PBLK		blkget(HBLK handle);
	/** Get a blocks size */
	virtual size_t		blkgetsize(PBLK pblk);			// get size of PBLK
	/** Find a block by handle */
	virtual	PBLK		blkfind(HBLK handle);
	/** Validate a block */
	virtual	Boolean 	blkvalidate(PBLK pblk);
	/** Set lock of a block */
	virtual	void		blksetlock(PBLK pblk);
	/** Unlock a block */
	virtual	Boolean		blksetunlock(PBLK pblk);
	/** Test, is block dirty */
	virtual	Boolean 	blkisdirty(PBLK pblk);
	/** Get block header */
	virtual	Boolean		blkgetheader(HBLK handle, PBLK pblk);
	/** write block headeer to file */
	virtual	Boolean		blkwriteheader(PBLK pblk);
	/** Read block header from file */
	virtual Boolean 	blkreadheader(HBLK handle, PBLK pblk);

	/** take a data pointer and returns its block */
	virtual PBLK 		getmfblk(void *data); 			// get PBLK from data pointer

	// file header funcions
	virtual	Boolean 	fileheaderwrite(_mfblk_header *p, unsigned long offset);
	virtual	Boolean 	fileheaderread(_mfblk_header *p, unsigned long offset);
	virtual	Boolean 	fileheadercreate(_mfblk_header *p, unsigned long offset);

	public:
	MFBlkfile(void);
	virtual ~MFBlkfile(void);

	// pblk functions	
	/** Allocate a block based on size, return block */
	virtual PBLK	pblkalloc(size_t size, enum BLK_REUSE method);
	/** Get a block from its handle, return block */
	virtual	PBLK	pblkget(HBLK handle);
	/** Lock a block, and mark it dirty, return block */
	virtual	PBLK	pblklock(HBLK handle);
	/** Unlock a block, write to disk, optionally release memory */
	virtual	void	pblkunlock(PBLK pblk, Boolean realease = TRUE);
	/** Free a block */
	virtual	void	pblkfree(PBLK pblk);
	/** Force a write of a block */
	virtual void	pblkwrite(PBLK pblk);

	// pointer functions
	/** Allocate a block based on size, return pointer */
	virtual void *	alloc(HBLK * handle, size_t size, enum BLK_REUSE method = BLK_BESTFIT);
	/** Get a block from its handle, return pointer */
	virtual void * 	get(HBLK handle);
	/** Lock a block handle, and mark it dirty, return pointer */
	virtual void * 	lock(HBLK handle);
	/** Unlock a block handle, write to disk, optionally release memory */
	virtual void	unlock(HBLK handle, Boolean release = TRUE);
	/** Free a block handle*/
	virtual	void	free(HBLK handle);

	// File finctions
	/** open a block file */
	virtual Boolean	use(char *fname); 
	/** close the block file */
	virtual void	close(void);
	/** writes all dirty blocks to memory, releases unlocked blocks */
	virtual	void	flush(void);		

	// Persistent information functions	
	/** Return block handle from master list */
	virtual	HBLK	getmaster(int ndx);
	/** store block handle in master list	 */
	virtual	Boolean	setmaster(int ndx, HBLK handle);
	/** get user type of block */
	virtual	UINT32	gettype(HBLK handle);
	/** set user type of block */
	virtual	UINT32	settype(HBLK handle, UINT32 type);
	/** get user property of block */
	virtual	UINT32	getprop(HBLK handle);
	/** set user property of block */
	virtual	UINT32	setprop(HBLK handle, UINT32 type);


	// Handle manipulation functions
	/** get handle from data pointer */
	virtual HBLK 	handle(void *data);
	/** get the size from the handle */
	virtual size_t	size(HBLK handle);	
	/** get number of blocks in list */
	virtual	int	numblocks(void);
	/** mark a block dirty --- causes a save */
	virtual	void	markdirty(HBLK handle, Boolean dirty=TRUE);

	// Friends and aquaintences
	friend class MFPersistent;
};
#endif


