/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef _FILE_

#define MEM_ALLOC	1024

#ifdef WIN32
#define NEEDS_RW_SYNC
#include <direct.h>
#endif

/** A basic file-like object */
class MFile
{
	protected:
	char * 	m_fname;

	public:
	virtual ~MFile()=0;
	virtual ssize_t	read(void *buf, size_t count)=0;
	virtual	ssize_t	write(void *buf, size_t count)=0;
	virtual ssize_t writes(char *buf);
	virtual ssize_t mfprintf(char *fmt, ...);
	virtual void flush(void);
	virtual FOFFSET seek(FOFFSET offset, int whence=SEEK_SET);	
	virtual FOFFSET tell();
	virtual void rewind();
	virtual void *getptr(int cb);
	virtual void *getptr(FOFFSET off, int cb);
	virtual void freeptr(void *p);
	virtual Boolean	stat(struct stat *buf);
	
// Just some inlines
	inline 	Boolean bread(void *buf, size_t count) { return read(buf,count) == (ssize_t)count; }
	inline 	Boolean bwrite(void *buf, size_t count) { return write(buf,count) == (ssize_t)count; }
};

/** A random access file class */
class MRandomFile : public MFile
{
	protected:
	Boolean			m_fCloned;

	public:
				MRandomFile();
	virtual int 		getbyte()=0;
	virtual Boolean 	eof()=0;
	virtual char *		gets(char *buf, int size)=0;
	virtual MRandomFile *	clone(void)=0;
};

/** a low level IO based file class */
class MIoFile : public MRandomFile
{
	protected:
	int	m_file;

	public:
	MIoFile(FILE *file=0);
	MIoFile(char *fname, int flags, mode_t mode);
	virtual ~MIoFile();
	virtual	Boolean open(char *filename, int flags, mode_t mode);
	virtual void close(Boolean fremove = FALSE);
	virtual ssize_t	read(void *buf, size_t count);
	virtual ssize_t	write(void *buf, size_t count);
	virtual FOFFSET seek(FOFFSET offset, int whence=SEEK_SET);	
	virtual FOFFSET tell();
	virtual void rewind();
	virtual int getbyte();
	virtual Boolean eof();
	virtual char *gets(char *buf, int size);
	virtual void flush(void);
	virtual	MRandomFile *clone(void);
	virtual Boolean	stat(struct stat *buf);
};

/** A MFile  wrapper for stdio files.  */
class MStdioFile : public MRandomFile
{
	protected:
	FILE *		m_file;
	unsigned	m_growsize;
	int		m_exec;

#ifdef NEEDS_RW_SYNC
	int		m_dir;
#endif

	public:	
	MStdioFile(FILE *file=0);
	MStdioFile(char *fname, char *how=NULL);
	virtual ~MStdioFile();
	virtual void SetFILE(FILE *f, Boolean fCloseOld=FALSE);
	virtual	Boolean open(char *filename, char *how=NULL);
	virtual void close(Boolean fremove = FALSE);
	virtual ssize_t	read(void *buf, size_t count);
	virtual ssize_t	write(void *buf, size_t count);
	virtual FOFFSET seek(FOFFSET offset, int whence=SEEK_SET);	
	virtual FOFFSET tell();
	virtual void rewind();
	virtual int getbyte();
	virtual Boolean eof();
	virtual char *gets(char *buf, int size);
	virtual void flush(void);
	virtual void d(void);
	virtual	MRandomFile *clone(void);
	virtual Boolean exec(char *command, char *mode=NULL);
};


/** A ram based random access file.  */
class MMemoryFile : public MRandomFile
{
	protected:
	MMutex		m_mutex;
	char *		m_buffer;
	unsigned 	m_offset;
	unsigned	m_bufsize;
	unsigned	m_growsize;
	unsigned 	m_end;

	virtual unsigned grow(unsigned cb);
	/** Creates a clone of object*/
	MMemoryFile(MMemoryFile *mmf);

	public:
	/** Creates a new object */
	MMemoryFile(unsigned int growsize=MEM_ALLOC);
	virtual ~MMemoryFile();
	virtual	ssize_t	read(void *buf, size_t count);
	virtual ssize_t	write(void *buf, size_t count);
	virtual FOFFSET seek(FOFFSET offset, int whence=SEEK_SET);	
	virtual FOFFSET tell();
	virtual void rewind();
	virtual int getbyte();
	virtual Boolean eof();
	virtual char *gets(char *buf, int size);
	virtual	char operator [] (unsigned offset);
	virtual void *getptr(int cb);
	virtual void *getptr(FOFFSET off, int cb);
	virtual void freeptr(void *p);
	virtual ssize_t dump(MFile *mf);
	virtual void trunc(void);
	virtual	MRandomFile *clone(void);
};

/** A read-only memory mapped file */
class MMapFile : public MMemoryFile
{
	protected:
#ifdef POSIX
	int m_fd;
#endif
#ifdef WIN32
	HANDLE			m_hfile;
	HANDLE			m_hmap;
#endif
 	unsigned grow(unsigned cb); 
	MMapFile(MMapFile *);
	public:
	MMapFile(char *filename);
	~MMapFile();
	virtual	MRandomFile *clone(void);
};

extern MStdioFile mferr;

#ifndef PATH_MAX
#ifdef MAX_PATH
#define PATH_MAX MAX_PATH
#else
#define PATH_MAX 512
#endif
#endif

#ifdef WIN32
#define OPEN 	_open
#define CLOSE 	_close
#define READ	_read
#define WRITE 	_write
#define LSEEK	_lseek
#define POPEN	winpopen
#define PCLOSE	winpclose
#define MKDIR(name, perm)	_mkdir(name)
#define S_IRUSR 0
#define S_IWUSR 0
extern FILE *winpopen(char *cmd, char *mode);
extern void winpclose(FILE *pf);
#endif

#ifdef POSIX
#define OPEN 	open
#define CLOSE 	close
#define READ	read
#define WRITE 	write
#define LSEEK	lseek
#define POPEN	popen
#define PCLOSE	pclose
#define MKDIR	mkdir
#endif

#endif
