/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef _MFPESRT_
#define _MFPERST_

/** Slight changes to the block file for persistent classes. 
NewObject should be overloaded and made to return objects based on type.
*/
class MFPersistfile : public MFBlkfile
{
	protected:
	void	mlistdelete(PBLK pblk);
	public:
	virtual ~MFPersistfile(void);
	/** overload this to create classes for your application. */
	virtual void * NewObject(int type, HBLK *phblk, ...);
};

/** A class that is persistent within a file.
By inheriting this class a class can be constructed which can test 
"isrestored()" and act one way if created new, and another if restored
from a block file.
Currently class types are not checked, but eventually they may be.
*/
class MFPersist
{
	private:
	MFPersistfile *	m_blkf;
	PBLK		m_pblk;
	Boolean		m_frestored;

	protected:
	/** Test if the class has been restored from a block file */
	Boolean		isrestored(void);
	/** Get user type of class */
	UINT32		gettype(void);
	/** Set user type of class */
	void		settype(UINT32 type);
	/** Get a property of the class */
	UINT32		getprop(void);
	/** Set a property of the class */
	void		setprop(UINT32 prop);
	/** Make class as dirty */
	void		setdirty(Boolean fdirty);
	/** Test for dirty */
	Boolean		isdirty(void);
	/** Set this object to be in the master list */
	void		setmaster(int ndx);
	/** Get blkfile handle of this object */
	HBLK		gethandle(void);
	/** the new operator */
	static void *	newop(size_t size, MFPersistfile *pblkf, HBLK *phblk);

	public:
	MFPersist(void);
	virtual	~MFPersist();
	void * operator new(size_t size, MFPersistfile *blkf, HBLK *phblk);
	void * operator new(size_t size, MFPersistfile *blkf, int ndx);
	void operator delete(void *p);
};

#endif

