/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef __MHASH__

typedef unsigned int HASH;

/** A hash table function 
    Uses lists and arrays
    default is to hash strings keys
*/
class MHash : public MNode, public MNodeHolder 
{
	protected:
	int		m_tsize;
	MList 		m_list;
	MTArray<MNode *>m_array;
	public:
			MHash(int tsize);
	virtual		~MHash();

	/** rehashes table to a new size */
	void 		Rehash(int tsize);

	/** find a node in the hash */
	MNode *		FindStr(HASH hash, char *key);
	MNode *		Find(HASH hash, NKey key);
	MNode *		Find(NKey key);

	/** Insert a node in to the hash */
	Boolean		Insert(HASH hash,MNode *insert);
	Boolean		Insert(MNode *insert);

	/** Remove a node from the hash */
	Boolean Remove(HASH hash,MNode *node);
	Boolean Remove(MNode *node);

	/** Hash tables are not sorted so ascending is ignored */
	void 	Enum(EnumFn fn, void *param1=NULL, void *param2=NULL, Boolean ascending=TRUE);

	/** gets hash code for a key. */
	virtual HASH 	chash(NKey key);

	/** hack strings for speed */
	HASH 	strhash(char *str)
	{

		register HASH hash;
		for(hash=0; *str; str++)
		{
			register HASH val =g_tolower_tbl[(unsigned char)*str];
			hash = (hash*val)+val; 
		}
		return (int)( hash % m_tsize);
	}

	int count(void) { return m_list.count() ; }

};
#endif
