/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to professionally license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef _MPLUGIN_
#define _MPLUGIN_

#define MPLUGIN_VER 0x43
#define NO_PLUGIN_FUNCTION (-1)

typedef int (*PlugFn)(void *p1, void *p2);

typedef struct _PluginFn
{
	PlugFn 	funct;
	char *	name;
}PLUGFN;

#define PLUGFN_OPEN(name)	static PLUGFN name [] = {
#define PLUGFN_DECL(name)	{ (PlugFn) name, #name },
#define PLUGFN_CLOSE		{0,0} };

typedef void * 	(*PlugInit)(int version, int mod, PLUGFN **imports, PLUGFN *exports ) ;
typedef int 	(*PlugHandler)(void *plugobject, int msg, void *p1, void *p2);
typedef void 	(*PlugShutdown)(void *plugobject);

#ifdef GCC	
#define PDECL	extern "C" 
#endif

#ifdef WIN32
#define PDECL	extern "C" __declspec(dllexport) 
#endif


#define FN_INIT(name) PDECL void * name##_INIT_ (int ver, int mod, PLUGFN **imports, PLUGFN *exports)
#define FN_HANDLER(name) PDECL int name##_HANDLER_ (void *pdata, int msg, void *p1, void *p2)
#define FN_SHUTDOWN(name) PDECL void name##_SHUTDOWN_ (void *pdata)

#ifdef POSIX
typedef void *HDLL;
#define __HDLL_DEFINED
#endif

#ifdef WIN32
typedef HMODULE HDLL;
#define __HDLL_DEFINED
#endif

#ifndef __HDLL_DEFINED
#error No plugin defined
#endif


class MPlugMgr;

class MPlugin
{
	friend class MPlugMgr;

	protected:
	PLUGFN *	m_imports;/** Function pointers imported from plugin */
	PLUGFN *	m_exports;/** Function pointers exported to plugin */
	HDLL		m_dll;
	void *		m_pdata;
	int 		m_ver;
	int		m_useCount;

	static void * 	def_init(int ver, int mod, PLUGFN **imports, PLUGFN *exports);
	static int 	def_handler(void *pdata, int msg, void *p1, void *p2);
	static void	def_shutdown(void *pdata);
	

	PlugInit	m_init;
	PlugHandler	m_handler;
	PlugShutdown	m_shutdown;

	/** Get pointer to a function, by name */
	void * 		getpfn(char *name);
	/** open a library by name */
	Boolean		openlib(char *name);
	/** close the library */
	void  		closelib(void);

	public:
	MPlugin(PLUGFN *m_exports = NULL);
	~MPlugin();
	int 		Open(char *module, char *name, int mod=0);
	void 		Close();
	PlugHandler	SetHandler(PlugHandler plh);
	PlugShutdown 	SetShutdown(PlugShutdown pls);
	inline int 	pcall(int msg, void *p1=NULL, void *p2=NULL){return (*m_handler)(m_pdata,msg,p1,p2);}
	/** get functions imported from the plugin */
	PLUGFN *	getimports();
	/** get functions exported to the plugin */
	PLUGFN *	getexports();
	int		getuse(void)	{return m_useCount; };
	int		incuse(void)	{return ++m_useCount;};
	void		SetExports(PLUGFN *exports);

};


class MPlugMgr
{
	protected:
	PLUGFN	*		m_exports;
	MAVL			m_functions;
	MTArray<MPlugin *>	m_plugins;

	public:
			MPlugMgr(PLUGFN *exports=NULL);
			~MPlugMgr();
	int 		MapFunctions(PLUGFN *plgf);
	MPlugin *	LoadPlugin(char *name, char *path);
	int 		CallPluginFn(char *name, void *p1, void *p2);
	PlugFn 		FindFn(char *name);
	void 		CloseAll(void);
	void		SetExports(PLUGFN *exports);
	Boolean 	LoadFunctionMod(char *optarg);
	void 		GetPluginInfo(char *optarg, char *name, char *path);
	PlugFn 		GetFn(char *fnname, PlugFn fnDefault);
};


#define PLUGMSG_VER	0
#define PLUGMSG_START	1

#endif

