/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef _MSERVER_
#define _MSERVER_

/** A threaded socket handler. */
class MMTServer  : public MThread
{
	protected:
	fd_set		m_fdsetSocket;	// an fd_set of all the sockets
	fd_set		m_fdsetRead;	// an fd_set of all the signaled sockets
	int 		m_count;	// count of signaled sockets in the fd_set.
	int		m_lastSock;	// Last socket to be used in the fd_set
	int		m_nsock;	// Number of sockets in the fdset
	int		m_nmaxSock;	// Largest sock id in fd_set for select
	MSock	*	m_sock;		// Listener socket
	MMutex		m_mutex;	// Mutex for dealing with fd_sets
	Boolean		m_fContinue;	// Set this to false to kill thread
	Boolean		m_fWait;	// Should get threads wait?
	
	void	RemoveSocket(int sock);
	void	AddSocket(int sock);
	void	threadLoop(void);
	int 	GetNextSocket(void);

	public:
		MMTServer(MSock *listener);
	MSock *	Accept(MSock *ms);
	void	Release(MSock *ms);
	void 	Finish(MSock *ms);
	int	NumWaiting(void) { return m_count; }
};

/** A threaded TCP/IP server */
class MServerThreadCore : public MThread
{

	protected:
	struct tms	m_tmsStart;
	struct tms	m_tmsEnd;

	clock_t		m_ctStart;
	clock_t		m_ctEnd;


	public:
	MServerThreadCore();
	~MServerThreadCore();

	inline clock_t	Ticks(int mult=1)
	{
#ifdef WIN32
		return GetTickCount();
#else
		struct tms tmsT;
		clock_t t = times(&tmsT);
		return mult*(t - m_ctStart);
#endif
	}
	
	virtual Boolean PreAccept(void);
	virtual void PrepSocket(MSock *ms);
	virtual Boolean HandleSocket(MSock *ms)=0;
	virtual void CloseSocket(MSock *ms);
	virtual void CleanUp(void);
};

class MServerThread : public MServerThreadCore
{
	protected:
	MSock *		m_listener;
	void threadLoop(void);

	public:
	MServerThread(MSock *listener);
	~MServerThread();
};


class MTServerThread : public MServerThreadCore
{
	protected:
	MMTServer *m_tsvr;
	void threadLoop(void);

	public:
	MTServerThread(MMTServer *tsvr);
	~MTServerThread();
};

#if 0

class MThreadedServer : public MThreadMgr
{
	protected:
	

	virtual	Boolean listen_wait(void);
	public:
	void Run(MSock *listener, int queue);
};

class MServerThThread : public MServerThreadCore
{
	protected:
	MMutex	m_mtx;
	MSock	m_sock;
	void threadLoop(void);
	
	public:

	MServerThThread();
	~MServerThThread();
	Boolean Accept(MSock *ms);
};

#endif

#endif
