/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef _MSQL_
#define _MSQL_

enum sql_errors
{
	SQLERR_OK,
	SQLERR_OK_DATA,
	SQLERR_OK_AFFECTED,
	SQLERR_OK_NONE,
	SQLERR_FAIL
};

#define SQLTYPE_UNDEFINED	(-1)

typedef void (*SQL_TRANSLATOR)(int col, int *type, unsigned int *cb);

typedef void * SQLRESULT;
class MSqlQueue;

/** A base class for SQL objects. Currently wraps ODBC and PostgreSQL */
class MSql
{
	friend class MSqlQueue;

	protected:
	int 	m_lifetime;
	int	m_qid;
	SQL_TRANSLATOR	m_translate;

	void	SetQid(int qid);
	int	GetQid(void);

	public:
	/** A printf SQL execution, uses printf to form query, and returns result object */
	SQLRESULT  ExecfResult(char *fmt, ...);

	/** A printf SQL execution, uses printf to form query, return status, no result */
	int 	   Execf(char *fmt, ...);
	/** Set translator, DB specific */
	SQL_TRANSLATOR SetTranslator(SQL_TRANSLATOR fn);


	/* Virtual functions which do not need to be replace
	   unless yor SQL needs something different
	*/
	virtual	Boolean		Begin(void);
	virtual	Boolean		Commit(void);
	virtual Boolean 	Abort();
	virtual	Boolean		ErrorToBool(int error);


	MSql();
	/* virtual functions to be replaced by implementer */
	virtual ~MSql()=0;
	/** Exec SQL statement and return the result object */
	virtual SQLRESULT  	ExecResult(char *sql)=0;
	/** Exec SQL statement, returns status, no results */
	virtual int 	   	Exec(char *szsql)=0;
	/** Exec SQL statement and return first value as an int */
	virtual Boolean 	ExecInt(char *sql, int *ret);
	/** Return number of columns in a result set */
	virtual	int 	   	ColumnCount(SQLRESULT res)=0;
	/** Return number of rows in a result set */
	virtual	int 	   	RowsCount(SQLRESULT res)=0;
	/** Get column index from a column name */
	virtual int 	   	ColumnIndex(SQLRESULT res, char *key)=0;
	/** Get the value from a result */
	virtual char *	   	ColumnValue(SQLRESULT res, int row, int ndx)=0;
	/** Get the value from a column based on name */
	virtual char *	   	ColumnName(SQLRESULT res, int ndx)=0;
	/** Free result pointer */
	virtual void	   	FreeResult(SQLRESULT sqlres)=0;
	/** Return type of data in result */
	virtual int	   	ColumnType(SQLRESULT sqlres, int ndx);

	/** Curser routines */
	virtual	Boolean		CursorCreate(char *name, char *sql, Boolean fBinary);	
	/** Fetch from cursor */
	virtual SQLRESULT  	CursorFetch(int num, char *curs)=0;
	/** Free result pointer returned from fetch*/
	virtual void	   	CursorFreeResult(SQLRESULT sqlres);
	/** Destroy Cursor */
	virtual void	   	CursorDestroy(char *curs);

	/** Does driver have limited lifetime */
	virtual int 		Lifetime(void);

	static MSql *Create(char *conninfo);
};

#ifdef POSTGRESQL
MSql *NewMPGSql(char *host, char *db, char *port = NULL);
#endif
#ifdef ODBC
MSql *NewMODBCSql(char *dbname, char *name, char *auth);
#endif

typedef struct _sql_entry
{
	MSql *sql;		// The SQL Object
	Boolean fInuse;		// Is it being used?
	int lifetime;		// Do not exceed this many uses
}QENTRY;


/** Create a thread safe queue of SQL connections */
class MSqlQueue
{
	protected:
	char 	*m_conninfo;
	int	m_num;
	int	m_count;
	MMutex	m_mutex;
	QENTRY	*m_entries;

	public:
		MSqlQueue();
		~MSqlQueue();

	/** Initilizes the queue */
	Boolean Init(char *conninfo, int nentries);

	/** Gets a SQL object from the queue. Will block */
	MSql *	Get();

	/** Releases SQL object after a Get() */
	void	Release(MSql *sql);

	/** Gets number of SQL connections allocated */
	int	GetCount(void);
};
	
#endif


