/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to commercially license this library, the author
    can be reached at info@mohawksoft.com
*/ 


#include "Phoenix.h"

#define MAX_SESSIONS	(1024*1024)-1
#define MAX_RLAB	128
#define MAX_LOCKTIME	60

class Session;
class SessionMgr;

#define SESSION_MUTEX
/** Used in MSession, this is the sesson object. It is an AVL tree which
 * holds session nodes. */
class Session : public MAVL
{
#ifdef SESSION_MUTEX
	MMutex		m_mtx;
#endif
	protected:
	int 		m_index;
	time_t		m_tcreate;
	time_t		m_tlast;
	time_t		m_locked;
	time_t		m_timeout;
	unsigned int 	m_flags;
	int 		m_lendata;

	/** Produces the hash value of a session name */
	int hash(char *name);

	public:
	Session(SessionMgr *smgr);
	~Session();
	/** Set the individual session timeout value */
	void	SetTimeout(time_t timeout);
	/** Get the individual session timeout value */
	time_t	GetTimeout(void);
	/** Get the last time this session was used */
	time_t	GetLastTime(void);
	/** Get the time this session was created */
	time_t	GetCreateTime(void);
	/** Set the last use time to now */
	void	SetLast(void);

	/** Get the amount of time this session has to live */
	time_t	GetTTL(time_t now, time_t default_age);
	/** Set a value in the tree */
	void	SetValue(char *name, char *value);
	/** Get a value in the tree */
	int	GetValue(char *name, char *data, int *datasize, Boolean updateTime=TRUE);
	/** Find a node by name and compare the value with it */
	int	CompareValue(char *name, char *value);
	/** Non virtual GetLastTime */
	time_t	last_used(void) { return m_tlast; }
	/** New operator */
	void *	operator new(size_t size, char * name);
	int	lock(void);
	/** Lock a session */
	Boolean unlock(int key);
	/** Test if a session is locked */
	Boolean islocked();

	/** Set a flag in a session */
	unsigned int SetFlag(unsigned int flag);
	/** Reset a flag in a session */
	unsigned int ResetFlag(unsigned int flag);
	unsigned int GetFlags(void);
	Boolean TestFlag(unsigned int flag);
	int	nextval(char *name, char *data);
	char *	SetSessionData(char *value, char *szfilePath, int cb=0);
	int	GetSessionData(char *data, int *datasize, char *szfilePath);
	Boolean	HasExpired(time_t now, time_t default_age);
	void	DeleteSessionData(char *szfilePath);

	char *	GetSessionDataReference(int *len);

	void mtxlock(void)	
	{
#ifdef SESSION_MUTEX
		m_mtx.lock();
#endif
	}
	void mtxunlock(void)	
	{
#ifdef SESSION_MUTEX
		m_mtx.unlock();
#endif
	}
};

class SessionMgr : public MHash
{
	protected:
	char *	m_szfilePath;

	public:
	SessionMgr(int size);
	void 	SetMaxLockTime(int t);
	void 	SetFilePath(char *szPath) { m_szfilePath = szPath; }
	char * 	GetFilePath(void) {return m_szfilePath; }
};

#define NODESESSION(node) ((Session *)node)
